from django.views.generic import DetailView
from django.utils.translation import gettext as _
from django.conf import settings
from django.http import Http404

from .models import Author, Blog, Tag, Post


class BlogDetailView(DetailView):
    language = None

    def get_language_code(self, url_path=None):
        default_lang = settings.LANGUAGES[0][0]
        if url_path is None:
            return default_lang
        lang_code = url_path.split("/")[1]
        return lang_code

    def set_language(self):
        self.language = self.get_language_code(self.request.META["PATH_INFO"])


class BlogView(BlogDetailView):
    template_name = "djcms_blog/blog.html"
    slug_field = "slug"
    slug_url_kwarg = "blog_slug"
    model = Blog

    def get_context_data(self, **kwargs):
        context = super(BlogView, self).get_context_data(**kwargs)
        return context

    def get(self, request, *args, **kwargs):
        self.set_language()
        self.object = self.get_object()
        lang_object = self.object.get_language_object(self.language)
        self.object.description = lang_object.description
        self.object.meta_title = lang_object.meta_title
        self.object.meta_description = lang_object.meta_description
        self.object.get_posts = self.object.get_posts()
        for post in self.object.get_posts:
            post_object = post.language_object(self.language)
            post.title = post_object.title
            post.description = post_object.description
            post.published_date = post_object.published_date
        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)


class TagView(BlogDetailView):
    template_name = "djcms_blog/tag.html"
    slug_field = "slug"
    slug_url_kwarg = "tag_slug"
    model = Tag

    def get_context_data(self, **kwargs):
        context = super(TagView, self).get_context_data(**kwargs)
        return context

    def get(self, request, *args, **kwargs):
        self.set_language()
        self.object = self.get_object()
        lang_object = self.object.get_language_object(self.language)
        self.object.description = lang_object.description
        self.object.meta_title = lang_object.meta_title
        self.object.meta_description = lang_object.meta_description
        self.object.get_posts = self.object.get_posts()
        for post in self.object.get_posts:
            post_object = post.language_object(self.language)
            post.title = post_object.title
            post.description = post_object.description
            post.published_date = post_object.published_date
        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)


class AutorView(BlogDetailView):
    template_name = "djcms_blog/author.html"
    slug_field = "slug"
    slug_url_kwarg = "author_slug"
    model = Author

    def get_context_data(self, **kwargs):
        context = super(AutorView, self).get_context_data(**kwargs)
        return context

    def get(self, request, *args, **kwargs):
        self.set_language()
        self.object = self.get_object()
        lang_object = self.object.get_language_object(self.language)
        self.object.bio = lang_object.bio
        self.object.get_posts = self.object.get_posts()
        for post in self.object.get_posts:
            post.title = post.language_object(self.language).title
            post.description = post.language_object(self.language).description
            post.published_date = post.language_object(self.language).published_date
        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)


class PostView(BlogDetailView):
    template_name = "djcms_blog/post.html"
    slug_field = "slug"
    slug_url_kwarg = "post_slug"
    model = Post

    def get_context_data(self, **kwargs):
        context = super(PostView, self).get_context_data(**kwargs)
        return context

    def get(self, request, *args, **kwargs):
        self.set_language()
        self.object = self.get_object()
        if not self.object.is_published:
            raise Http404("Not found")

        lang_object = self.object.language_object(self.language)
        self.object.title_id = lang_object.id
        self.object.title = lang_object.title
        self.object.body = lang_object.body
        self.object.publisher_public = lang_object.publisher_public
        self.object.published = lang_object.published
        self.object.publisher_draft = lang_object.publisher_draft
        self.object.published_date = lang_object.published_date
        self.object.meta_title = lang_object.meta_title
        self.object.meta_description = lang_object.meta_description
        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)


class PostDraftView(PostView):
    def get_object(self, queryset=None):

        if queryset is None:
            queryset = self.get_queryset()

        pk = self.kwargs.get(self.pk_url_kwarg)
        slug = self.kwargs.get(self.slug_url_kwarg)
        if pk is not None:
            queryset = queryset.filter(pk=pk)

        if slug is not None and (pk is None or self.query_pk_and_slug):
            slug_field = self.get_slug_field()
            queryset = queryset.filter(**{slug_field: slug, "publisher_is_draft": True})

        if pk is None and slug is None:
            raise AttributeError(
                "Generic detail view %s must be called with "
                "either an object pk or a slug." % self.__class__.__name__
            )
        try:
            obj = queryset.get()
        except queryset.model.DoesNotExist:
            raise Http404(
                _("No %(verbose_name)s found matching the query")
                % {"verbose_name": queryset.model._meta.verbose_name}
            )
        return obj



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings
import simplemde.fields


class Migration(migrations.Migration):

    dependencies = [migrations.swappable_dependency(settings.AUTH_USER_MODEL)]

    operations = [
        migrations.CreateModel(
            name="Author",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        primary_key=True,
                        serialize=False,
                        auto_created=True,
                    ),
                ),
                ("cover", models.ImageField(upload_to="author_cover")),
                ("image", models.ImageField(upload_to="image")),
                ("slug", models.CharField(max_length=140)),
                ("location", models.CharField(max_length=100)),
                ("website", models.URLField(max_length=100)),
                ("facebook_profile", models.URLField(max_length=100)),
                ("twitter_profile", models.URLField(max_length=100)),
                (
                    "block_header",
                    simplemde.fields.SimpleMDEField(
                        max_length=10000, blank=True, null=True
                    ),
                ),
                (
                    "block_footer",
                    simplemde.fields.SimpleMDEField(
                        max_length=10000, blank=True, null=True
                    ),
                ),
                (
                    "user",
                    models.OneToOneField(
                        related_name="author_profile", to=settings.AUTH_USER_MODEL
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="AuthorBio",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        primary_key=True,
                        serialize=False,
                        auto_created=True,
                    ),
                ),
                ("bio", simplemde.fields.SimpleMDEField(max_length=255)),
                (
                    "language",
                    models.CharField(
                        max_length=15,
                        db_index=True,
                        choices=[("en", "en"), ("es", "es")],
                    ),
                ),
                ("author", models.ForeignKey(to="djcms_blog.Author")),
            ],
        ),
        migrations.CreateModel(
            name="Blog",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        primary_key=True,
                        serialize=False,
                        auto_created=True,
                    ),
                ),
                ("title", models.CharField(max_length=140)),
                ("slug", models.CharField(max_length=140, db_index=True)),
                ("cover", models.ImageField(upload_to="blog_cover")),
                (
                    "block_header",
                    simplemde.fields.SimpleMDEField(
                        max_length=10000, blank=True, null=True
                    ),
                ),
                (
                    "block_footer",
                    simplemde.fields.SimpleMDEField(
                        max_length=10000, blank=True, null=True
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="BlogTitle",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        primary_key=True,
                        serialize=False,
                        auto_created=True,
                    ),
                ),
                (
                    "language",
                    models.CharField(
                        max_length=15,
                        db_index=True,
                        choices=[("en", "en"), ("es", "es")],
                    ),
                ),
                ("title", models.CharField(max_length=140)),
                ("description", simplemde.fields.SimpleMDEField(max_length=255)),
                (
                    "meta_title",
                    simplemde.fields.SimpleMDEField(
                        max_length=70, blank=True, null=True
                    ),
                ),
                (
                    "meta_description",
                    simplemde.fields.SimpleMDEField(
                        max_length=156, blank=True, null=True
                    ),
                ),
                ("blog", models.ForeignKey(to="djcms_blog.Blog")),
            ],
        ),
        migrations.CreateModel(
            name="Post",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        primary_key=True,
                        serialize=False,
                        auto_created=True,
                    ),
                ),
                ("title", models.CharField(max_length=255)),
                ("slug", models.CharField(max_length=140)),
                (
                    "cover",
                    models.ImageField(blank=True, null=True, upload_to="post_cover"),
                ),
                ("author", models.ForeignKey(to="djcms_blog.Author")),
                ("blog", models.ForeignKey(to="djcms_blog.Blog")),
            ],
        ),
        migrations.CreateModel(
            name="PostTitle",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        primary_key=True,
                        serialize=False,
                        auto_created=True,
                    ),
                ),
                ("title", models.CharField(max_length=255)),
                (
                    "language",
                    models.CharField(
                        max_length=15,
                        db_index=True,
                        choices=[("en", "en"), ("es", "es")],
                    ),
                ),
                ("description", simplemde.fields.SimpleMDEField(max_length=80000)),
                ("body", simplemde.fields.SimpleMDEField(max_length=80000)),
                (
                    "meta_title",
                    simplemde.fields.SimpleMDEField(
                        max_length=70, blank=True, null=True
                    ),
                ),
                (
                    "meta_description",
                    simplemde.fields.SimpleMDEField(
                        max_length=156, blank=True, null=True
                    ),
                ),
                ("published", models.BooleanField(default=False)),
                (
                    "publisher_is_draft",
                    models.BooleanField(db_index=True, default=True, editable=False),
                ),
                ("created", models.DateTimeField(auto_now_add=True)),
                ("modified", models.DateTimeField(auto_now=True)),
                ("published_date", models.DateTimeField(blank=True, null=True)),
                ("post", models.ForeignKey(to="djcms_blog.Post")),
                (
                    "publisher_public",
                    models.OneToOneField(
                        null=True,
                        editable=False,
                        related_name="publisher_draft",
                        to="djcms_blog.PostTitle",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Tag",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        primary_key=True,
                        serialize=False,
                        auto_created=True,
                    ),
                ),
                ("cover", models.ImageField(upload_to="tag_cover")),
                ("name", models.CharField(max_length=140)),
                ("slug", models.CharField(max_length=140)),
                (
                    "color",
                    models.CharField(
                        max_length=14,
                        choices=[
                            ("red", "red"),
                            ("orange", "orange"),
                            ("yellow", "yellow"),
                            ("olive", "olive"),
                            ("green", "green"),
                            ("teal", "teal"),
                            ("blue", "blue"),
                            ("violet", "violet"),
                            ("purple", "purple"),
                            ("pink", "pink"),
                            ("brown", "brown"),
                            ("grey", "grey"),
                            ("black", "black"),
                        ],
                    ),
                ),
                (
                    "meta_title",
                    simplemde.fields.SimpleMDEField(
                        max_length=70, blank=True, null=True
                    ),
                ),
                (
                    "meta_description",
                    simplemde.fields.SimpleMDEField(
                        max_length=156, blank=True, null=True
                    ),
                ),
                ("blog", models.ForeignKey(to="djcms_blog.Blog")),
            ],
        ),
        migrations.CreateModel(
            name="TagTitle",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        primary_key=True,
                        serialize=False,
                        auto_created=True,
                    ),
                ),
                (
                    "language",
                    models.CharField(
                        max_length=15,
                        db_index=True,
                        choices=[("en", "en"), ("es", "es")],
                    ),
                ),
                ("name", models.CharField(max_length=140)),
                ("description", simplemde.fields.SimpleMDEField(max_length=200)),
                (
                    "meta_title",
                    simplemde.fields.SimpleMDEField(
                        max_length=70, blank=True, null=True
                    ),
                ),
                (
                    "meta_description",
                    simplemde.fields.SimpleMDEField(
                        max_length=156, blank=True, null=True
                    ),
                ),
                ("tag", models.ForeignKey(to="djcms_blog.Tag")),
            ],
        ),
        migrations.AddField(
            model_name="post",
            name="tag",
            field=models.ManyToManyField(to="djcms_blog.Tag"),
        ),
        migrations.AlterUniqueTogether(
            name="tagtitle", unique_together=set([("tag", "language")])
        ),
        migrations.AlterUniqueTogether(
            name="posttitle",
            unique_together=set([("post", "language", "publisher_is_draft")]),
        ),
        migrations.AlterUniqueTogether(
            name="blogtitle", unique_together=set([("blog", "language")])
        ),
    ]



from django.conf.urls import url
from functools import wraps
from django.utils.decorators import available_attrs
from django.views.decorators.cache import cache_page
from djcms_blog.settings import DJCMS_BLOG_CACHE_TIME

from . import views


def cache_for_anonim(timeout):
    def decorator(view_func):
        @wraps(view_func, assigned=available_attrs(view_func))
        def _wrapped_view(request, *args, **kwargs):

            if request.user.is_staff:
                return (view_func)(request, *args, **kwargs)
            else:
                return cache_page(timeout)(view_func)(request, *args, **kwargs)

        return _wrapped_view

    return decorator


urlpatterns = [
    url(
        r"^(?P<blog_slug>[\w-]+)/$",
        cache_for_anonim(DJCMS_BLOG_CACHE_TIME)(views.BlogView.as_view()),
        name="blog-main",
    ),
    url(
        r"^author/(?P<author_slug>[\w-]+)/$",
        cache_for_anonim(DJCMS_BLOG_CACHE_TIME)(views.AutorView.as_view()),
        name="author-main",
    ),
    url(
        r"^(?P<blog_slug>[\w-]+)/tag/(?P<tag_slug>[\w-]+)/$",
        cache_for_anonim(DJCMS_BLOG_CACHE_TIME)(views.TagView.as_view()),
        name="tag-main",
    ),
    url(
        r"^(?P<blog_slug>[\w-]+)/(?P<post_slug>[\w-]+)/$",
        cache_for_anonim(DJCMS_BLOG_CACHE_TIME)(views.PostView.as_view()),
        name="post-detail",
    ),
    url(
        r"^draft/(?P<blog_slug>[\w-]+)/(?P<post_slug>[\w-]+)/$",
        views.PostDraftView.as_view(),
        name="draft-post-detail",
    ),
]



from django.db import models
from django.contrib.auth.models import User
from simplemde.fields import SimpleMDEField
from datetime import datetime
from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils import timezone

from .utils import expire_page


def expire_blog_post(post):
    post_url = reverse(
        "post-detail",
        kwargs={"blog_slug": post.post.blog.slug, "post_slug": post.post.slug},
    )
    expire_page(post_url)
    blog_url = reverse("blog-main", kwargs={"blog_slug": post.post.blog.slug})
    expire_page(blog_url)
    author_url = reverse("author-main", kwargs={"author_slug": post.post.author.slug})
    expire_page(author_url)


class Author(models.Model):
    user = models.OneToOneField(User, related_name="author_profile")
    cover = models.ImageField(upload_to="author_cover")
    image = models.ImageField(upload_to="image")
    slug = models.CharField(max_length=140)
    location = models.CharField(max_length=100)
    website = models.URLField(max_length=100)
    facebook_profile = models.URLField(max_length=100)
    twitter_profile = models.URLField(max_length=100)
    block_header = SimpleMDEField(max_length=10000, blank=True, null=True)
    block_footer = SimpleMDEField(max_length=10000, blank=True, null=True)

    def __str__(self):
        return self.user.email

    def get_language_object(self, language):
        language_object = AuthorBio.objects.filter(
            author=self, language=language
        ).first()
        if language_object:
            return language_object

    def has_language(self, language):
        if AuthorBio.objects.filter(author=self, language=language).first():
            return True
        return False

    def get_posts(self):
        return Post.objects.author_posts(author=self)

    def get_post_count(self):
        return Post.objects.author_posts(author=self).count()

    def get_name(self):
        return "{} {}".format(self.user.first_name, self.user.last_name)


class AuthorBio(models.Model):
    author = models.ForeignKey(Author, db_index=True)
    bio = SimpleMDEField(max_length=255)
    language = models.CharField(
        max_length=15, db_index=True, choices=settings.LANGUAGES
    )

    def __str__(self):
        return "{} {}".format(self.author, self.language)

    def get_posts(self):
        return Post.objects.blog_posts(blog=self)


class Blog(models.Model):
    title = models.CharField(max_length=140)
    slug = models.CharField(max_length=140, db_index=True)
    cover = models.ImageField(upload_to="blog_cover")
    block_header = SimpleMDEField(max_length=10000, blank=True, null=True)
    block_footer = SimpleMDEField(max_length=10000, blank=True, null=True)

    def __str__(self):
        return self.title

    def get_posts(self):
        return Post.objects.blog_posts(blog=self)

    def has_language(self, language):
        if BlogTitle.objects.filter(blog=self, language=language).first():
            return True
        return False

    def get_language_object(self, language):
        language_object = BlogTitle.objects.filter(blog=self, language=language).first()
        if language_object:
            return language_object


class BlogTitle(models.Model):
    blog = models.ForeignKey(Blog, db_index=True)
    language = models.CharField(
        max_length=15, db_index=True, choices=settings.LANGUAGES
    )
    title = models.CharField(max_length=140)
    description = SimpleMDEField(max_length=255)
    meta_title = SimpleMDEField(max_length=70, blank=True, null=True)
    meta_description = SimpleMDEField(max_length=156, blank=True, null=True)

    class Meta:
        unique_together = ("blog", "language")

    def __str__(self):
        return self.title


class Tag(models.Model):
    COLOR_CHOICES = (
        ("red", "red"),
        ("orange", "orange"),
        ("yellow", "yellow"),
        ("olive", "olive"),
        ("green", "green"),
        ("teal", "teal"),
        ("blue", "blue"),
        ("violet", "violet"),
        ("purple", "purple"),
        ("pink", "pink"),
        ("brown", "brown"),
        ("grey", "grey"),
        ("black", "black"),
    )
    blog = models.ForeignKey(Blog, db_index=True)
    cover = models.ImageField(upload_to="tag_cover")
    name = models.CharField(max_length=140)
    slug = models.CharField(max_length=140)
    color = models.CharField(max_length=14, choices=COLOR_CHOICES)
    meta_title = SimpleMDEField(max_length=70, blank=True, null=True)
    meta_description = SimpleMDEField(max_length=156, blank=True, null=True)

    def __str__(self):
        return self.name

    def has_language(self, language):
        if TagTitle.objects.filter(tag=self, language=language).first():
            return True
        return False

    def get_posts(self):
        return Post.objects.published_tag(tag=self)

    def get_post_count(self):
        return Post.objects.published_tag(tag=self).count()

    def get_language_object(self, language):
        language_object = TagTitle.objects.filter(tag=self, language=language).first()
        if language_object:
            return language_object


class TagTitle(models.Model):
    tag = models.ForeignKey(Tag, db_index=True)
    language = models.CharField(
        max_length=15, db_index=True, choices=settings.LANGUAGES
    )
    name = models.CharField(max_length=140)
    description = SimpleMDEField(max_length=200)
    meta_title = SimpleMDEField(max_length=70, blank=True, null=True)
    meta_description = SimpleMDEField(max_length=156, blank=True, null=True)

    class Meta:
        unique_together = ("tag", "language")

    def __str__(self):
        return self.name


class PostManager(models.Manager):
    def published(self):
        posts = PostTitle.objects.filter(
            published=True, publisher_is_draft=False
        ).values_list("post", flat=True)
        return Post.objects.filter(id__in=posts)

    def published_tag(self, tag):
        return self.published().filter(tag__in=[tag])

    def blog_posts(self, blog):
        return self.published().filter(blog=blog)

    def author_posts(self, author):
        return self.published().filter(author=author)


class Post(models.Model):
    blog = models.ForeignKey(Blog, db_index=True)
    title = models.CharField(max_length=255)
    slug = models.CharField(max_length=140)
    cover = models.ImageField(upload_to="post_cover", blank=True, null=True)
    author = models.ForeignKey(Author, db_index=True)
    tag = models.ManyToManyField(Tag)

    objects = PostManager()

    def __str__(self):
        return self.title

    def get_tags(self):
        return self.tag.all()

    def language_object(self, language):
        language_object = PostTitle.objects.filter(
            post=self, language=language, publisher_is_draft=False, published=True
        ).first()
        if language_object:
            return language_object
        return PostTitle.objects.filter(
            post=self, publisher_is_draft=False, published=True
        ).first()

    def get_language_object(self, language):
        language_object = PostTitle.objects.filter(
            post=self, language=language, publisher_is_draft=True
        ).first()
        if language_object:
            return language_object

    def has_language(self, language):
        if PostTitle.objects.filter(post=self, language=language).first():
            return True
        return False

    def is_published(self):
        if PostTitle.objects.filter(
            post=self, publisher_is_draft=False, published=True
        ).first():
            return True
        return False


class PostTitle(models.Model):
    post = models.ForeignKey(Post, db_index=True)
    title = models.CharField(max_length=255)
    language = models.CharField(
        max_length=15, db_index=True, choices=settings.LANGUAGES
    )
    description = SimpleMDEField(max_length=80000)
    body = SimpleMDEField(max_length=80000)
    meta_title = SimpleMDEField(max_length=70, blank=True, null=True)
    meta_description = SimpleMDEField(max_length=156, blank=True, null=True)
    published = models.BooleanField(blank=True, default=False)
    publisher_is_draft = models.BooleanField(
        default=True, editable=False, db_index=True
    )
    publisher_public = models.OneToOneField(
        "self",
        on_delete=models.CASCADE,
        related_name="publisher_draft",
        null=True,
        editable=False,
    )
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)
    published_date = models.DateTimeField(blank=True, null=True)

    class Meta:
        unique_together = ("post", "language", "publisher_is_draft")

    def save(self, *args, **kwargs):
        print("Values to save", args, kwargs)
        super(PostTitle, self).save(*args, **kwargs)

    def edited(self):
        if self.publisher_public:
            if all(
                (
                    self.publisher_public.post == self.post,
                    self.publisher_public.title == self.title,
                    self.publisher_public.language == self.language,
                    self.publisher_public.description == self.description,
                    self.publisher_public.body == self.body,
                    self.publisher_public.meta_title == self.meta_title,
                    self.publisher_public.meta_description == self.meta_description,
                )
            ):
                return False
            return True
        return False

    def create_public_post(self):
        publisher_public = PostTitle(
            post=self.post,
            title=self.title,
            language=self.language,
            description=self.description,
            body=self.body,
            meta_title=self.meta_title,
            meta_description=self.meta_description,
            publisher_is_draft=False,
            published=True,
            published_date=datetime.now(tz=timezone.utc),
        )
        publisher_public.save()
        self.publisher_public = publisher_public
        self.published = True
        self.publisher_edited = False
        self.published_date = datetime.now(tz=timezone.utc)
        self.save()

    def publish(self):
        if self.publisher_public is None:
            self.create_public_post()
            expire_blog_post(self)
            return True
        else:
            if self.edited():
                publisher_public = self.publisher_public
                self.publisher_public = None
                self.create_public_post()
                expire_blog_post(self)
                publisher_public.delete()
                return True
        return False

    def unpublish(self):
        publisher_public = self.publisher_public
        self.publisher_public = None
        publisher_public.delete()
        self.published = False
        self.published_date = None
        expire_blog_post(self)
        self.save()



from django import template
from djcms_blog.models import Blog
from django.core.urlresolvers import reverse
from django.conf import settings

register = template.Library()


@register.simple_tag(name="blog_url")
def get_blog_url():
    blog = Blog.objects.first()
    if blog:
        return reverse("blog-main", kwargs={"blog_slug": blog.slug})
    if settings.djcms_blog_DEFAULT_URL:
        return settings.djcms_blog_DEFAULT_URL
    return "#"



from django.contrib import admin
from django.core.urlresolvers import reverse
from django.contrib import messages
from django.conf import settings
from django.utils.translation import gettext_lazy as _

from .models import Author, AuthorBio, Blog, BlogTitle, Post, PostTitle, Tag, TagTitle


add_string = _("Add")
view_string = _("View")


def make_published(modeladmin, request, queryset):
    for post in queryset:
        published = post.publish()
        if published:
            messages.success(request, "{} published".format(post.title))
        else:
            messages.error(request, "'{}' does not have changes".format(post.title))


make_published.short_description = "Publish selected posts"


def make_unpublished(modeladmin, request, queryset):
    for post in queryset:
        post.unpublish()


make_unpublished.short_description = "Unpublish selected posts"


class PostAdmin(admin.ModelAdmin):
    list_display = ["title", "is_published", "blog"]
    filter_horizontal = ("tag",)
    prepopulated_fields = {"slug": ("title",)}

    def is_published(self, obj):
        return obj.is_published()

    is_published.boolean = True


for lang in settings.LANGUAGES:

    def get_tag():
        lang_code = lang[0]

        def lang_tag(self, obj):
            lang_object = obj.get_language_object(lang_code)
            if lang_object:
                return '<a href="/admin/djcms_blog/posttitle/{}/" class="button">{} {}</a>'.format(
                    lang_object.id, view_string, lang_code.upper()
                )
            return '<a class="button" href="/admin/djcms_blog/posttitle/add/">{} {}</a>'.format(
                add_string, lang_code.upper()
            )

        lang_tag.short_description = lang_code.upper()
        lang_tag.allow_tags = True
        lang_tag.__name__ = lang_code
        return lang_tag

    setattr(PostAdmin, lang[0], get_tag())
    PostAdmin.list_display.append(lang[0])


class PostTitleAdmin(admin.ModelAdmin):
    list_display = ("title", "is_edited", "post", "language", "published")
    actions = [make_published, make_unpublished]
    exclude = ("published", "published_date")

    def is_edited(self, obj):
        return obj.edited()

    is_edited.boolean = True

    def view_on_site(self, obj):
        if obj.published:
            return reverse(
                "post-detail",
                kwargs={"blog_slug": obj.post.blog.slug, "post_slug": obj.post.slug},
            )
        else:
            return reverse(
                "draft-post-detail",
                kwargs={"blog_slug": obj.post.blog.slug, "post_slug": obj.post.slug},
            )

    def get_model_perms(self, request):
        """
        Return empty perms dict thus hiding the model from admin index.
        """
        return {}

    def get_queryset(self, request):
        qs = super(PostTitleAdmin, self).get_queryset(request)
        return qs.filter(publisher_is_draft=True)


class BlogAdmin(admin.ModelAdmin):
    list_display = ["title", "slug", "post_count"]
    prepopulated_fields = {"slug": ("title",)}

    def post_count(self, obj):
        return obj.get_posts().count()

    def view_on_site(self, obj):
        return reverse("blog-main", kwargs={"blog_slug": obj.slug})


for lang in settings.LANGUAGES:

    def get_tag():
        lang_code = lang[0]

        def lang_tag(self, obj):
            lang_object = obj.get_language_object(lang_code)
            if lang_object:
                return '<a href="/admin/djcms_blog/blogtitle/{}/" class="button">{} {}</a>'.format(
                    lang_object.id, view_string, lang_code.upper()
                )
            return '<a class="button" href="/admin/djcms_blog/blogtitle/add/">{} {}</a>'.format(
                add_string, lang_code.upper()
            )

        lang_tag.short_description = lang_code.upper()
        lang_tag.allow_tags = True
        lang_tag.__name__ = lang_code
        return lang_tag

    setattr(BlogAdmin, lang[0], get_tag())
    BlogAdmin.list_display.append(lang[0])


class BlogTitleAdmin(admin.ModelAdmin):
    list_display = ("blog", "title", "language")

    def view_on_site(self, obj):
        return reverse("blog-main", kwargs={"blog_slug": obj.blog.slug})

    def get_model_perms(self, request):
        """
        Return empty perms dict thus hiding the model from admin index.
        """
        return {}


class TagAdmin(admin.ModelAdmin):
    list_display = ["name", "slug", "color", "post_count", "blog"]
    prepopulated_fields = {"slug": ("name",)}

    def post_count(self, obj):
        return obj.get_posts().count()

    def view_on_site(self, obj):
        return reverse(
            "tag-main", kwargs={"blog_slug": obj.blog.slug, "tag_slug": obj.slug}
        )


for lang in settings.LANGUAGES:

    def get_tag():
        lang_code = lang[0]

        def lang_tag(self, obj):
            lang_object = obj.get_language_object(lang_code)
            if lang_object:
                return '<a href="/admin/djcms_blog/authorbio/{}/" class="button">{} {}</a>'.format(
                    lang_object.id, view_string, lang_code.upper()
                )
            return '<a class="button" href="/admin/djcms_blog/tagtitle/add/">{} {}</a>'.format(
                add_string, lang_code.upper()
            )

        lang_tag.short_description = lang_code.upper()
        lang_tag.allow_tags = True
        lang_tag.__name__ = lang_code
        return lang_tag

    setattr(TagAdmin, lang[0], get_tag())
    TagAdmin.list_display.append(lang[0])


class TagTitleAdmin(admin.ModelAdmin):
    list_display = ("tag", "language", "name")

    def view_on_site(self, obj):
        return reverse(
            "tag-main",
            kwargs={"blog_slug": obj.tag.blog.slug, "tag_slug": obj.tag.slug},
        )

    def get_model_perms(self, request):
        """
        Return empty perms dict thus hiding the model from admin index.
        """
        return {}


class AuthorAdmin(admin.ModelAdmin):
    list_display = ["user", "slug", "location", "website", "post_count"]

    def post_count(self, obj):
        return obj.get_posts().count()

    def view_on_site(self, obj):
        return reverse("author-main", kwargs={"author_slug": obj.slug})


class AuthorBioAdmin(admin.ModelAdmin):
    list_display = ["author", "language"]

    def post_count(self, obj):
        return obj.get_posts().count()

    def view_on_site(self, obj):
        return reverse("author-main", kwargs={"author_slug": obj.author.slug})

    def get_model_perms(self, request):
        """
        Return empty perms dict thus hiding the model from admin index.
        """
        return {}


for lang in settings.LANGUAGES:

    def get_tag():
        lang_code = lang[0]

        def lang_tag(self, obj):
            lang_object = obj.get_language_object(lang_code)
            if lang_object:
                return '<a href="/admin/djcms_blog/authorbio/{}/" class="button">{} {}</a>'.format(
                    lang_object.id, view_string, lang_code.upper()
                )
            return '<a class="button" href="/admin/djcms_blog/authorbio/add/">{} {}</a>'.format(
                add_string, lang_code.upper()
            )

        lang_tag.short_description = lang_code.upper()
        lang_tag.allow_tags = True
        lang_tag.__name__ = lang_code
        return lang_tag

    setattr(AuthorAdmin, lang[0], get_tag())
    AuthorAdmin.list_display.append(lang[0])


admin.site.register(Author, AuthorAdmin)
admin.site.register(AuthorBio, AuthorBioAdmin)
admin.site.register(Blog, BlogAdmin)
admin.site.register(BlogTitle, BlogTitleAdmin)
admin.site.register(PostTitle, PostTitleAdmin)
admin.site.register(Post, PostAdmin)
admin.site.register(Tag, TagAdmin)
admin.site.register(TagTitle, TagTitleAdmin)
